package com.zsw.starter;

import cn.hutool.core.thread.ThreadUtil;
import com.zsw.starter.handler.ClientHandler;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.json.JsonObjectDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.timeout.IdleStateHandler;
import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.TimeUnit;

@Slf4j
public class Client {

    private void start(int i) throws InterruptedException {
        NioEventLoopGroup group = new NioEventLoopGroup();
        Bootstrap bootstrap = new Bootstrap();
        bootstrap.group(group)
                .channel(NioSocketChannel.class)
                .option(ChannelOption.TCP_NODELAY, true)
                .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 3000)
                .handler(new ChannelInitializer<SocketChannel>() {
                    protected void initChannel(SocketChannel ch) {
                        // 注册名称
                        ch.pipeline().addLast("idleStateHandler", new IdleStateHandler(60, 60, 60, TimeUnit.SECONDS));
                        ch.pipeline().addLast(new JsonObjectDecoder());
                        ch.pipeline().addLast(new StringEncoder());
                        ch.pipeline().addLast(new ClientHandler(i));
                    }
                });
        ChannelFuture channelFuture = bootstrap.connect("127.0.0.1", 8899);
        channelFuture.sync();
        channelFuture.channel().closeFuture().sync();
    }

    public void run(){
        for (int i = 0; i < 10; i++) {
            // try {
            //     Thread.sleep(200);
            // } catch (InterruptedException e) {
            //     throw new RuntimeException(e);
            // }
            int finalI = i;
            ThreadUtil.execAsync(()->{
                log.info("client:{} ready", finalI);
                try {
                    this.start(finalI);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            });

        }


    }
}
