package com.zsw.starter;

import com.zsw.starter.handler.ServerHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioChannelOption;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.json.JsonObjectDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.timeout.IdleStateHandler;
import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.TimeUnit;

@Slf4j
public class Server {

    private void start() throws InterruptedException {
        NioEventLoopGroup bossGroup = new NioEventLoopGroup(1);
        //new 一个工作线程组
        NioEventLoopGroup workGroup = new NioEventLoopGroup(200);
        ServerBootstrap bootstrap = new ServerBootstrap()
                .group(bossGroup, workGroup)
                .channel(NioServerSocketChannel.class)
                .childHandler(new ChannelInitializer<SocketChannel>() {
                    @Override
                    protected void initChannel(SocketChannel socketChannel) throws Exception {
                        ChannelPipeline pipline = socketChannel.pipeline();
                        pipline.addLast("heart",new IdleStateHandler(5,5,10, TimeUnit.SECONDS));
                        // pipline.addFirst(new JsonObjectDecoder());
                        pipline.addLast(new StringEncoder());
                        pipline.addLast(new ServerHandler());
                    }
                })
                .localAddress(8899)
                .option(ChannelOption.SO_BACKLOG, 1024)
                .childOption(NioChannelOption.SO_KEEPALIVE, true);
        ChannelFuture future = bootstrap.bind().sync();
        log.info("打印服务启动,端口:{} ",8899);
    }

    public void run(){
        try {
            this.start();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }
}
